#include "KarbonCalligraphyOptionWidget.h"

#include <KLocale>
#include <KComboBox>
#include <KGlobal>
#include <KConfigGroup>
#include <KDebug>
#include <KMessageBox>

#include <QtGui/QSpinBox>
#include <QtGui/QDoubleSpinBox>
#include <QtGui/QVBoxLayout>
#include <QtGui/QLabel>
#include <QtGui/QPushButton>
#include <QtGui/QInputDialog>

/*
Profiles are saved in karboncalligraphyrc

In the group "General", profile is the name of profile used

Every profile is described in a group, the name of which is "ProfileN"
Starting to count from 0 onwards
(NOTE: the index in profiles is different from the N)

Default profiles are added by the function addDefaultProfiles(), once they
have been added, the entry defaultProfilesAdded in the "General" group is
set to true

TODO: add a reset defaults option?
*/

KarbonCalligraphyOptionWidget::KarbonCalligraphyOptionWidget()
    : changingProfile(false), detailsShowed(true)
{
    QVBoxLayout *layout = new QVBoxLayout( this );

    QLabel *profileLabel = new QLabel( i18n("Profile:"), this );
    layout->addWidget( profileLabel );
    comboBox = new KComboBox( this );
    layout->addWidget( comboBox );

    QHBoxLayout *widthLayout = new QHBoxLayout( this );
    QLabel *widthLabel = new QLabel( i18n( "Width" ), this );
    widthBox = new QDoubleSpinBox;
    widthBox->setRange( 0.0, 1000.0 );
    widthLayout->addWidget( widthLabel );
    widthLayout->addWidget( widthBox );
    layout->addLayout( widthLayout );

    detailsButton = new QPushButton( "", this );
    layout->addWidget( detailsButton );

    details = new QWidget( this );
    QVBoxLayout *detailsLayout = new QVBoxLayout( details );
    
    QHBoxLayout *thinningLayout = new QHBoxLayout( this );
    QLabel *thinningLabel = new QLabel( i18n( "Thinning" ), this );
    thinningBox = new QDoubleSpinBox;
    thinningBox->setRange( -1.0, 1.0 );
    thinningBox->setSingleStep( 0.1 );
    thinningLayout->addWidget( thinningLabel );
    thinningLayout->addWidget( thinningBox );
    detailsLayout->addLayout( thinningLayout );

    QHBoxLayout *angleLayout = new QHBoxLayout( this );
    QLabel *angleLabel = new QLabel( i18n( "Angle" ), this );
    angleBox = new QSpinBox;
    angleBox->setRange( 0, 180 );
    angleLayout->addWidget( angleLabel );
    angleLayout->addWidget( angleBox );
    detailsLayout->addLayout( angleLayout );

    QHBoxLayout *fixationLayout = new QHBoxLayout( this );
    QLabel *fixationLabel = new QLabel( i18n( "Fixation" ), this );
    fixationBox = new QDoubleSpinBox;
    fixationBox->setRange( 0.0, 1.0 );
    fixationBox->setSingleStep( 0.1 );
    fixationLayout->addWidget( fixationLabel );
    fixationLayout->addWidget( fixationBox );
    detailsLayout->addLayout( fixationLayout );

    QHBoxLayout *massLayout = new QHBoxLayout( this );
    QLabel *massLabel = new QLabel( i18n( "Mass" ), this );
    massBox = new QDoubleSpinBox;
    massBox->setRange( 0.0, 20.0 );
    massBox->setDecimals( 1 );
    massLayout->addWidget( massLabel );
    massLayout->addWidget( massBox );
    detailsLayout->addLayout( massLayout );

    QHBoxLayout *dragLayout = new QHBoxLayout( this );
    QLabel *dragLabel = new QLabel( i18n( "Drag" ), this );
    dragBox = new QDoubleSpinBox;
    dragBox->setRange( 0.0, 1.0 );
    dragBox->setSingleStep( 0.1 );
    dragLayout->addWidget( dragLabel );
    dragLayout->addWidget( dragBox );
    detailsLayout->addLayout( dragLayout );

    saveButton = new QPushButton( i18n("Save profile as..."), this );
    detailsLayout->addWidget( saveButton );

    removeButton = new QPushButton( i18n("Remove profile"), this );
    detailsLayout->addWidget( removeButton );

    layout->addWidget( details );
    layout->addStretch( 1 );

    toggleDetails();

    addDefaultProfiles(); // if they are already added does nothing
    loadProfiles();
    createConnections();
}

KarbonCalligraphyOptionWidget::~KarbonCalligraphyOptionWidget()
{
    qDeleteAll( profiles );
}

/*void KarbonCalligraphyOptionWidget::customProfile()
{
    addProfile( createProfile(i18n("Custom")) );
}*/

void KarbonCalligraphyOptionWidget::emitAll()
{
    emit widthChanged( widthBox->value() );
    emit thinningChanged( thinningBox->value() );
    emit angleChanged( angleBox->value() );
    emit fixationChanged( fixationBox->value() );
    emit massChanged( massBox->value() );
    emit dragChanged( dragBox->value() );
}

void KarbonCalligraphyOptionWidget::loadProfile( const QString &name )
{
    // write the new profile in the config file
    KConfig config( KGlobal::mainComponent(), "karboncalligraphyrc" );
    KConfigGroup generalGroup( &config, "General" );
    generalGroup.writeEntry( "profile", name );
    config.sync();

    // and load it
    loadCurrentProfile();

    // don't show Current if it isn't selected
    if (name != i18n("Current")) {
        removeProfile( i18n("Current") );
    }
}

void KarbonCalligraphyOptionWidget::updateCurrentProfile()
{
    if ( ! changingProfile )
        saveProfile("Current");
}

void KarbonCalligraphyOptionWidget::saveProfileAs()
{
    QString name;

    // loop until a valid name is entered or the user cancelled
    while (1) {
        bool ok;
        name = QInputDialog::getText( this,
                i18n("Profile name"),
                i18n("Please insert the name by which "
                "you want to save this profile:"),
                QLineEdit::Normal, QString(), &ok );
        if ( ! ok ) return;

        if ( name.isEmpty() || name == i18n("Current") )
        {
            KMessageBox::sorry( this,
                        i18n("Sorry, the name you entered is invalid!"),
                        i18nc("invalid profile name", "Invalid name!") );
            // try again
            saveProfileAs();
            continue; // ask again
        }

        if ( profiles.contains(name) )
        {
            int ret = KMessageBox::warningYesNo( this,
                        i18n("A profile with that name already exists!\n"
                                "Do you want to overwrite it?") );

            if ( ret == KMessageBox::Yes )
                break; // exit while loop (save profile)
            // else ask again
        }
        else
        {
            // the name is valid
            break; // exit while loop (save profile)
        }
    }

    saveProfile(name);
}

void KarbonCalligraphyOptionWidget::removeProfile()
{
    removeProfile( comboBox->currentText() );
}

void KarbonCalligraphyOptionWidget::toggleDetails()
{
    if ( detailsShowed )
    {
        details->hide();
        detailsButton->setText( i18n("Show details >>") );
    }
    else
    {
        details->show();
        detailsButton->setText( i18n("Hide details <<") );
    }

    detailsShowed = !detailsShowed;
}


/******************************************************************************
 ************************* Convenience Functions ******************************
 ******************************************************************************/

void KarbonCalligraphyOptionWidget::createConnections()
{
    connect ( comboBox, SIGNAL(currentIndexChanged(const QString &)),
              SLOT(loadProfile(const QString &)) );


    connect( widthBox, SIGNAL(valueChanged(double)),
             SIGNAL(widthChanged(double)) );

    connect( thinningBox, SIGNAL(valueChanged(double)),
             SIGNAL(thinningChanged(double)) );

    connect( angleBox, SIGNAL(valueChanged(int)),
             SIGNAL(angleChanged(int)) );

    connect( fixationBox, SIGNAL(valueChanged(double)),
             SIGNAL(fixationChanged(double)) );

    connect( massBox, SIGNAL(valueChanged(double)),
             SIGNAL(massChanged(double)) );

    connect( dragBox, SIGNAL(valueChanged(double)),
             SIGNAL(dragChanged(double)) );


    connect( widthBox, SIGNAL(valueChanged(double)),
             SLOT(updateCurrentProfile()) );

    connect( thinningBox, SIGNAL(valueChanged(double)),
             SLOT(updateCurrentProfile()) );

    connect( angleBox, SIGNAL(valueChanged(int)),
             SLOT(updateCurrentProfile()) );

    connect( fixationBox, SIGNAL(valueChanged(double)),
             SLOT(updateCurrentProfile()) );

    connect( massBox, SIGNAL(valueChanged(double)),
             SLOT(updateCurrentProfile()) );

    connect( dragBox, SIGNAL(valueChanged(double)),
             SLOT(updateCurrentProfile()) );


    connect( saveButton, SIGNAL(clicked()), SLOT(saveProfileAs()) );
    connect( removeButton, SIGNAL(clicked()), SLOT(removeProfile()) );
    connect( detailsButton, SIGNAL(clicked()), SLOT(toggleDetails()) );
}

void KarbonCalligraphyOptionWidget::addDefaultProfiles()
{
    // check if the profiles where already added
    KConfig config( KGlobal::mainComponent(), "karboncalligraphyrc" );
    KConfigGroup generalGroup( &config, "General" );

    if ( generalGroup.readEntry( "defaultProfilesAdded", false ) )
        return;

    KConfigGroup profile0( &config, "Profile0" );
    profile0.writeEntry( "name", i18n("Mouse") );
    profile0.writeEntry( "width", 30.0 );
    profile0.writeEntry( "thinning", 0.2 );
    profile0.writeEntry( "angle", 30 );
    profile0.writeEntry( "fixation", 1.0 );
    profile0.writeEntry( "mass", 3.0 );
    profile0.writeEntry( "drag", 0.7 );

    KConfigGroup profile1( &config, "Profile1" );
    profile1.writeEntry( "name", i18n("Graphics Pen") );
    profile1.writeEntry( "width", 50.0 );
    profile1.writeEntry( "thinning", 0.2 );
    profile1.writeEntry( "angle", 30 );
    profile1.writeEntry( "fixation", 1.0 );
    profile1.writeEntry( "mass", 1.0 );
    profile1.writeEntry( "drag", 0.9 );

    generalGroup.writeEntry( "profile", i18n("Mouse") );
    generalGroup.writeEntry( "defaultProfilesAdded", true );

    config.sync();
}


void KarbonCalligraphyOptionWidget::loadProfiles()
{
    KConfig config( KGlobal::mainComponent(), "karboncalligraphyrc" );

    // load profiles as long as they are present
    int i = 0;
    while (1) // forever
    {
        KConfigGroup profileGroup( &config, "Profile" + QString::number(i) );
        // invalid profile, assume we reached the last one
        if ( ! profileGroup.hasKey("name") )
            break;

        Profile *profile = new Profile;
        profile->index = 1;
        profile->name =     profileGroup.readEntry( "name", QString() );
        profile->width =    profileGroup.readEntry( "width", 30.0 );
        profile->thinning = profileGroup.readEntry( "thinning", 0.2 );
        profile->angle =    profileGroup.readEntry( "angle", 30 );
        profile->fixation = profileGroup.readEntry( "fixation", 0.0 );
        profile->mass =     profileGroup.readEntry( "mass", 3.0 );
        profile->drag =     profileGroup.readEntry( "drag", 0.7 );

        profiles.insert( profile->name, profile );
        ++i;
    }

    foreach (const QString &name, profiles.keys())
    {
        comboBox->addItem( name );
    }

    loadCurrentProfile();
}

void KarbonCalligraphyOptionWidget::loadCurrentProfile()
{
    KConfig config( KGlobal::mainComponent(), "karboncalligraphyrc" );
    KConfigGroup generalGroup( &config, "General" );
    QString currentProfile = generalGroup.readEntry( "profile", QString() );

    // find the index needed by the comboBox
    int index = profilePosition( currentProfile );

    if ( currentProfile == "" || index < 0 )
    {
        kError() << "invalid karboncalligraphyrc!!";
        return;
    }

    kDebug() << comboBox->currentIndex() << index;
    comboBox->setCurrentIndex( index );

    Profile *profile = profiles[currentProfile];

    changingProfile = true;
    widthBox->setValue( profile->width );
    thinningBox->setValue( profile->thinning );
    angleBox->setValue( profile->angle );
    fixationBox->setValue( profile->fixation );
    massBox->setValue( profile->mass );
    dragBox->setValue( profile->drag );
    changingProfile = false;
}

void KarbonCalligraphyOptionWidget::saveProfile( const QString &name )
{

    Profile *profile = new Profile;
    profile->name = name;
    profile->width = widthBox->value();
    profile->thinning = thinningBox->value();
    profile->angle = angleBox->value();
    profile->fixation = fixationBox->value();
    profile->mass = massBox->value();
    profile->drag = dragBox->value();

    if ( profiles.contains(name) ) {
        // there is already a profile with the same name, overwrite
        profile->index = profiles[name]->index;
        profiles.insert( name, profile );
    } else {
        // it is a new profile
        profile->index = profiles.count();
        profiles.insert( name, profile );
        // ad the profile to the combobox
        comboBox->insertItem( profilePosition(name), name );
    }

    comboBox->setCurrentIndex( profilePosition(name) );

    KConfig config( KGlobal::mainComponent(), "karboncalligraphyrc" );
    QString str = "Profile" + QString::number( profile->index );
    KConfigGroup profileGroup( &config, str );

    profileGroup.writeEntry( "name", name );
    profileGroup.writeEntry( "width", profile->width );
    profileGroup.writeEntry( "thinning", profile->thinning );
    profileGroup.writeEntry( "angle", profile->angle );
    profileGroup.writeEntry( "fixation", profile->fixation );
    profileGroup.writeEntry( "mass", profile->mass );
    profileGroup.writeEntry( "drag", profile->drag );

    KConfigGroup generalGroup( &config, "General" );
    generalGroup.writeEntry( "profile", name );

    config.sync();
}

void KarbonCalligraphyOptionWidget::removeProfile(const QString &name)
{
    int index = profilePosition(name);
    if ( index < 0 ) return; // no such profile

    // remove the file from the config file
    KConfig config( KGlobal::mainComponent(), "karboncalligraphyrc" );
    int deletedIndex = profiles[name]->index;
    QString deletedGroup = "Profile" + QString::number( deletedIndex );
    config.deleteGroup( deletedGroup );
    config.sync();

    // and from profiles
    profiles.remove(name);

    comboBox->removeItem(index);

    // now in the config file there is value ProfileN missing,
    // where N = configIndex, so put the last one there
    if ( profiles.isEmpty() ) return;

    int lastN = -1;
    Profile *profile = 0; // profile to be moved, will be the last one
    foreach ( Profile *p, profiles )
    {
        if ( p->index > lastN )
        {
            lastN = p->index;
            profile = p;
        }
    }

    Q_ASSERT(profile != 0);

    // do nothing if the deleted group was the last one
    if ( deletedIndex > lastN ) return;

    QString lastGroup = "Profile" + QString::number( lastN );
    config.deleteGroup( lastGroup );

    KConfigGroup profileGroup( &config, deletedGroup );
    profileGroup.writeEntry( "name", profile->name );
    profileGroup.writeEntry( "width", profile->width );
    profileGroup.writeEntry( "thinning", profile->thinning );
    profileGroup.writeEntry( "angle", profile->angle );
    profileGroup.writeEntry( "fixation", profile->fixation );
    profileGroup.writeEntry( "mass", profile->mass );
    profileGroup.writeEntry( "drag", profile->drag );
    config.sync();

    profile->index = deletedIndex;
}

int KarbonCalligraphyOptionWidget::profilePosition( const QString &profileName )
{
    int res = 0;
    foreach (const QString &name, profiles.keys())
    {
        if (name == profileName)
            return res;
        ++res;
    }
    return -1;
}
